# This is translated from Rust to Python.

# Constants for rate limit and API endpoints
RATE_LIMIT_SLEEP_20_PER_2 = 100  # Milliseconds
HOST = "https://www.okx.com"
REST_TICKER = "/api/v5/market/ticker?instId={}"
REST_INSTRUMENTS = "/api/v5/public/instruments?instType={}"
REST_FUNDING_RATE = "/api/v5/public/funding-rate?instId={}"
REST_CANDLE_STICK = "/api/v5/market/mark-price-candles?limit=100&instId={instId}&bar={bar}"



import json
import logging
from oqclib.utils.http_util import send_post_request_with_retries

logger = logging.getLogger(__name__)

class LarkMsg():
    def __init__(self, keys_dict):
        self.keys_dict = keys_dict
        self.request_url = 'https://open.larksuite.com/open-apis/bot/v2/hook/'

    def show_lark(self):
        '''
        show available lark config
        :return: lark keys avaliable in config
        '''
        print(self.keys_dict.keys())

    def send_msg(self, robot, msg): 
        '''
        send lark message
        :param msg: message to send
        :return:
        '''
        key = self.keys_dict[robot]

        mentioned_list = ['@all']

        textJSON = [[{'tag': 'text', 'text': msg}]]

        headers = {
            'Content-Type': 'application/json; charset=utf-8'
        }

        textMsg = {
            'msg_type': 'post',
            'content': {'post': {'en_us': {'content': textJSON}}}
        }

        se = json.dumps(textMsg)
        #r = requests.post(self.request_url + key, data=se, headers=headers)
        url = self.request_url + key
        r = send_post_request_with_retries(url, se, 3, headers=headers)
        logger.info("Sending lark resp: {}".format(r.text))


    def send_card(self, robot, card_body: dict):
        '''
        send lark message card
        :param title: lark card title
        :param msg: message to send
        :param robot: lark key name as defined in lark_config
        :return:
        '''
        key = self.keys_dict[robot]
        headers = {
            'Content-Type': 'application/json; charset=utf-8'
        }
        textMsg = {
            "msg_type": "interactive",
            "card": card_body
        }
        se = json.dumps(textMsg)
        #r = requests.post(self.request_url + key, data=se, headers=headers)
        url = self.request_url + key
        r = send_post_request_with_retries(url, se, 3, headers=headers)
        logger.info("Sending lark resp: {}".format(r.text))



import os

def has_gui():
    # Check if DISPLAY environment variable is set (common in Unix-like systems)
    if os.name == 'posix' and 'DISPLAY' not in os.environ:
        return False
    # Windows and macOS are generally running under a GUI
    elif os.name == 'nt' or sys.platform == 'darwin':
        return True
    return False




def format_time_period(minutes:int) -> str:
    if minutes < 60:
        return f"{minutes}m"  # Minutes
    elif minutes == 60:
        return "1H"  # 1 Hour
    elif minutes < 1440:
        hours = minutes // 60
        return f"{hours}H"  # Hours
    elif minutes == 1440:
        return "1D"  # 1 Day
    elif minutes < 10080:
        days = minutes // 1440
        return f"{days}D"  # Days
    elif minutes < 43200:
        weeks = minutes // 10080
        return f"{weeks}W"  # Weeks
    else:
        months = minutes // 43200
        return f"{months}M"  # Months

import asyncio
import aiohttp
import time
from datetime import datetime

async def fetch_candlesticks(client, symbol, bar_period, look_back_minutes):
    result = []
    now_ms = int(time.time() * 1000)
    from_ms = now_ms - look_back_minutes * 1000 * 60

    while True:
        to_ms = from_ms + 1000 * 60 * 100
        # url = f"{HOST}{REST_CANDLE_STICK.replace('{instId}', symbol).replace('{bar}', bar_period)}&before={to_ms}&after={from_ms}"
        
        print(f"Requesting {url} from {datetime.fromtimestamp(from_ms/1000)} to {datetime.fromtimestamp(to_ms/1000)}")
        async with client.get(url) as response:
            response_data = await response.json()
            if response_data['code'] == '0':  # Assuming '0' indicates success
                response_data['data'].reverse()
                result.extend(response_data['data'])
        
        if to_ms >= now_ms:
            break
        from_ms = to_ms
        await asyncio.sleep(RATE_LIMIT_SLEEP_20_PER_2 / 1000)  # Convert milliseconds to seconds

    return result

# Example usage
async def main():
    async with aiohttp.ClientSession() as client:
        candles = await fetch_candlesticks(client, 'BTC-USD', '1m', 60)
        print(candles)

if __name__ == '__main__':
    asyncio.run(main())



from datetime import datetime, timedelta


def seconds_to_next_interval(dt: datetime, interval_seconds: int) -> float:
    # Calculate total minutes since midnight
    seconds_since_midnight = (dt.hour * 60 + dt.minute) * 60 + dt.second

    # Find the next multiple of the interval
    next_interval_second = ((seconds_since_midnight // interval_seconds) + 1) * interval_seconds

    # Calculate the time of the next interval
    next_interval_time = datetime(dt.year, dt.month, dt.day, 0, 0) + timedelta(seconds=next_interval_second)

    # Calculate the number of seconds until the next interval
    seconds_to_next = (next_interval_time - dt).total_seconds()

    return seconds_to_next


def is_midnight() -> bool:
    # Get the current local time
    now = datetime.now()

    # Get the hour component of the current time
    hour = now.hour

    return hour < 7


if __name__ == '__main__':
    print(seconds_to_next_interval(datetime.now(), 3600 * 12))
    print(is_midnight())
